import React, { useEffect, useState } from 'react';
import { GetRoleUser } from '@/services/platform/workflow';
import {
  Form,
  Modal,
  Input,
  Checkbox,
  Button,
  Pagination,
  Spin,
  Divider,
} from 'antd';
import Tree from '@/components/ExpendableTree';
import { UserOutlined } from '@ant-design/icons';
import styles from '../../../workflow.less';

const CheckboxGroup = Checkbox.Group;

const RoalChoose = props => {
  const { onSubumit, handleCancel, visible } = props;
  const [form] = Form.useForm();
  const [currentSelectId, setCurrentSelectId] = useState(''); // 当前选择节点Id
  const [currentRoalName, setCurrentRoalName] = useState(''); // 当前选择节点名称
  const [treeData, setTreeData] = useState([]); // 左侧树数据
  const [checkedList, setCheckedList] = useState([]); // 选中的数据
  const [plainOptions, setPlainOptions] = useState([]); // 默认的的数据
  const [indeterminate, setIndeterminate] = useState(false); // 全选框的有选择但是没有全选的样式
  const [checkAll, setCheckAll] = useState(false); // 全选框全选样式
  const [total, setTotal] = useState(); // 角色人员条数
  const [loading, setLoading] = useState(false); // 加载时的loading
  const [currentPage, setCurrentPage] = useState(1);
  // 初始化
  useEffect(() => {
    form.resetFields();
    if (visible) {
      setCurrentSelectId('');
      setPlainOptions([]);
      setCheckedList([]);
      setIndeterminate(false);
      setCheckAll(false);
      setCurrentPage(1);
      getData('', 0);
    } else {
      setTreeData([]);
    }
  }, [visible]);
  // 获取表单回显
  const getData = (roleName, pageIndex, pageSize) => {
    roleName = roleName || '';
    setLoading(true);
    GetRoleUser({ roleName, pageIndex, pageSize }).then(res => {
      setLoading(false);
      if (res.code === 0) {
        setTreeData(res.data.data);
        setTotal(res.data.count);
      }
    });
  };
  // 提交表单
  const onFinish = () => {
    form.validateFields().then(validate => {
      if (validate) {
        setCurrentSelectId('');
        setPlainOptions([]);
        setCheckedList([]);
        setIndeterminate(false);
        setCheckAll(false);
        setCurrentPage(1);
        getData(validate.roleName, 1);
      }
    });
  };
  // 处理属性结构
  const mapTree = item => {
    const obj = { ...item };
    return {
      title: obj.name,
      key: obj.id,
      userList: obj.userList,
    };
  };
  // 选择树
  const onSelect = (selectedKeys, e) => {
    setCurrentRoalName(e.node.title);
    if (selectedKeys[0]) {
      // 选择后对复选框的用户进行重新赋值
      let userList = [...e.node.userList];
      let options = userList.map(item => ({
        label: item.name,
        value: item.id,
      }));
      setCurrentSelectId(selectedKeys[0]);
      setPlainOptions(options);
      setCheckedList([]);
      setIndeterminate(false);
      setCheckAll(false);
    }
  };
  // 监听分页
  const paginationChange = (page, pageSize) => {
    // 分页时清空复选框
    setCurrentSelectId('');
    setPlainOptions([]);
    setCheckedList([]);
    setIndeterminate(false);
    setCheckAll(false);
    setCurrentPage(page);
    let roleName = form.getFieldValue('roleName');
    getData(roleName, page, pageSize);
  };
  // 复选框全选监听
  const onCheckAllChange = e => {
    let allOptions = plainOptions.map(item => item.value);
    setCheckedList(e.target.checked ? allOptions : []);
    setIndeterminate(false);
    setCheckAll(e.target.checked);
  };
  // 复选框监听事件
  const onChange = list => {
    setCheckedList(list);
    setIndeterminate(!!list.length && list.length < plainOptions.length);
    setCheckAll(list.length === plainOptions.length);
  };
  // 保存选择的角色列表
  const onSave = () => {
    // 默认承办人字符串拼接
    let DeafaultUserName = '';
    checkedList.forEach((item, index) => {
      plainOptions.forEach(element => {
        if (item === element.value) {
          if (index === checkedList.length - 1) {
            DeafaultUserName += `${element.label}`;
          } else {
            DeafaultUserName += `${element.label},`;
          }
        }
      });
    });
    // 存入要插入的数据
    let obj = {
      roleName: currentRoalName,
      roleId: currentSelectId,
      defaultUserId: checkedList,
      defauletUserName: DeafaultUserName,
    };
    onSubumit(obj);
  };
  return (
    <Modal
      title="角色用户列表"
      visible={visible}
      onOk={onSave}
      width="860px"
      onCancel={handleCancel}
      maskClosable={false}
      destroyOnClose
    >
      <div className={styles.pageContent}>
        {/* 角色选择树 */}
        <div className={styles.roleContent}>
          {/* 角色搜索 */}
          <div className={styles.header}>
            <Form form={form} layout="inline" onFinish={onFinish}>
              <Form.Item name="roleName">
                <Input
                  prefix={<UserOutlined className="site-form-item-icon" />}
                  placeholder="RoleName"
                  allowClear
                />
              </Form.Item>
              <Form.Item>
                <Button type="primary" htmlType="submit">
                  搜索
                </Button>
              </Form.Item>
            </Form>
          </div>
          {/* 角色树 */}
          <Spin spinning={loading}>
            <div className={styles.treeContent}>
              <span style={{ marginLeft: '28px', fontWeight: 700 }}>
                角色列表
              </span>
              <Divider style={{ margin: '10px 0' }} />
              <Tree
                blockNode
                autoExpandParent
                onSelect={onSelect}
                selectedKeys={[currentSelectId]}
                treeData={treeData.map(item => mapTree(item))}
              />
            </div>
          </Spin>
          {/* 分页 */}
          <div className={styles.footer}>
            <Pagination
              size="small"
              total={total}
              showSizeChanger
              defaultCurrent={1}
              current={currentPage}
              onChange={paginationChange}
            />
          </div>
        </div>
        {/* 用户列表 */}
        <div className={styles.userContent}>
          <div className={styles.userCheckBox}>
            <Checkbox
              indeterminate={indeterminate}
              onChange={onCheckAllChange}
              checked={checkAll}
            >
              <span style={{ fontWeight: 700 }}>用户列表</span>
            </Checkbox>
            <Divider style={{ margin: '10px 0' }} />
            <div className={styles.checkContent}>
              <CheckboxGroup
                options={plainOptions}
                value={checkedList}
                onChange={onChange}
              />
            </div>
          </div>
        </div>
      </div>
    </Modal>
  );
};
export default RoalChoose;
